# -*- sh -*-
################################################################################
## This is the script defining level 1. Scripts for other levels are very similar
## to this one, using only different maps, and placing game objects (GOBs) at 
## different locations.
##
## The role of the level script is to
## - define the resources to be loaded
## - including gob-description scripts
## - instanciating gobs, and possibly completing their initial state.
## - defining level-scoped actions such as "level cleared" or "game over".

###############################################################################
## load level resources
print "loading level1"

# bg% represent the tiled world. the GameScript class assumes the convention that
# bg0 is the layer with which the player will interact. The behaviour of bg1 and
# bg2 is currently not configurable (a second layer that is aligned with bg0, and
# a parallax background

bg0.load "spritey.spr"           # specifies the tileset to be used
bg0.map "assault1.map" 128 128   # specifies the map file that describes the world.
# ''128 128'' is the initial position of the camera in this map (in pixels).

print "activating background"
bg1.map = bg0.map:1 128 128
# the level editor conveniently lets us edit the two 'playfield' layers together:
# the one (used on bg0) that is rendered _in front of_ the hero, and the one (bg1)
# rendered _behind_ the hero. ''bg1.map = bg0.map:1'' is a way to tell "reuse the
# same map file as the one loaded for bg0, but using the second layer.

bg2.load "woodsb.spr"
bg2.map embedded 128 128
# the background scene has both tileset and tile map within a single .spr file.
# this is typical from images converted using imlib2spr.pl ''bg%.map embedded''
# is a way to tell the GameScript to use that .spr-embedded map for a particular
# background.

print "loading sprites"
spr.load "spritey.spr":1
# The NintendoDS has separated video memory for backgrounds and sprites. Yet, for
# convenience, the Sprite Editor of the dsgametools projects allows a single file
# to hold both tileset and spriteset in a single unit of edition. The ''default''
# tileset was loaded for bg0 (and automatically reused by bg1) the '':1'' tail of
# the spr.load command instructs the gamescript to extract the "extra" spriteset
# for sprite VRAM content.

# the spritey.spr has objects laid out as following :
# pages : [0]Bilou, [1]Bilou again, [2]cliff&bush&apple, [3]anims & woodworm
#         [4]appleman&fruitbat, [5] caves, [6]tree, [7]leaves, [8]



##################################################################################
# game counters 
# A large part of the top-level game logic is handled through the persistent 
# game counters. Each counter can be set an initial value in the top-level script
# (here) with ''let c% = <value>'' statement (where % is the number of the counter),
#  and can only be incremented or decremented by GOBs later on in the game. 


let c2 = 5  # hitpoints are clearly to be implemented through a counter

# counter #3 stores the amount of lives. 
c3.action = level(assault0.cmd,D) # Game Over
# ''c%.action = <action name>(<parameters>,<expression>)'' allows us to launch a
#   specific action when the counter reaches 0. Here, it loads the menu level
#   Note that c3 was initialised in that assault0.cmd file, and that it must not
#   be reset in level files.


# Woodworm
# As you can guess, the ''input "filename.cmd"'' statement makes the parser pause
#  reading of the current file and starts reading of the specified file. An important
#  aspect is that the sub-file receives its own evaluation context as far as animations
#  and states are concerned. That is, state0 in worm.cmd is separated from state0 in
#  funghi, etc. To be able to instanciate GOBs on the top-level script, the ''import''
#  statement lets you retrieve the k first states of the last sub-file you ''input''ed
#  as states for the top-level state.
input "worm.cmd"
import state0..1

using sound(5, 60) as 0

#  this ''import state0..1'' statement, thus, make state0 of woodworm being state0 for
#  level 1, and state1 of woodworm being known as state1 for the level as well. 
#  other states (e.g. stunned woodworm) aren't directly accessible and thus you can't
#  instanciate a stunned woodworm in your level.

input "abilou.cmd"
import state 4..7
# with ''import state 4..7'' just after ''input "bilou.cmd"'' statement, we reuse the
#  four first states of the hero's state machine (state0: falling facing right, 
#  state1: falling facing right, state2: winning dance, state3 : death animation)
#  being known as state4, state5, state6 and state7 respectively in the level script.


gob0 :state4 (128, 128) h      # instanciate our hero GOB as gob#0.
focus=gob0

# the ''gob%GOBNO% :state%STATENO% (<coords>)'' statement places an additional
# GOB at a specific position in the level and with the given initial state. The 
# terminating 'h' character stands for 'HERO'. 
# Giving a cast (HERO/EVIL) to a GOB is required so that it can receive collisions.

#######################################################################################
##  SPECIAL ANIMATIONS

# level clear when #Apples == 0
#  this animation is managed through an action attached to a counter the same way 
#  we attached 'level(...)' to C3. Here, the action is "launch track", managed by
#  the TrackGun c++ class in guns.cxx, that selects a part of the musical soundtrack
c4.action = track(38) { 
@4: setstate(gob0:=state6)
@16: fade(219)
@32: level(assault2.cmd)
}
# the TrackGun has another interesting feature: in the .XM module file we use, each
#  track (= pattern) contains 64 rows and we can attach a specific action to rows.
#  with ''@%ROW%: <action-name>(<parameters>)'' statements.
# row 4 : we change the behaviour of the hero (gob0) so that he starts the victory
#  dance (see GobstateGun class)
# row 16 : we clear the screen. '219' is the number of the IBM-extended character 
#  that is fully opaque, which we use here for clearing.
# row 32 : the new level is actually launched. No other statement can be read 
#  beyond that one.


# you're dead
c2.action = track(37) { 
@0: setstate(gob0:=state7)
@16: fade(219)
@32: level(assault1.cmd,d0d3) # try again
}
#  the 'death' animation works in a similar way, using another soundtrack and being
#  attached to another counter (C2, which we use as hitpoints counter). The special
#  part is the ''d0d3'' expression that is executed after ASSAULT1 has been scheduled
#  as the next level.
#  'd0' decrement counter #0, which is the current level counter, cancelling the 
#  level progress and making us stick at the difficulty level we just failed.
#  'd3' decrements the number of lives. Note that if that makes the number of lives
#  reach 0, the action ''level(assault0.cmd)'' is triggetred, and the level #0 will
#  effectively be loaded.

print "yo"


input "funghi.cmd"
import state3

input "berrybat.cmd"
import state2
using shgob(state0 is e:=D) as 12
input "berryshoot.cmd"
import state 12..13

print "setting counters"
c0.print "level "

#######################################################################################
## The increasing difficulty of the game as level are cleared is handled through counters
## C0 : the level counter. It is reset to 0 in ASSAULT0 and incremented in every other level.
## C7 : the number of applemen on screen, set by the statement below to 5 * LEVEL.
## C4 : the number of applemen to beat in this level.
let c:= (i0 v0 5* :7)
let c:= (d8 v7 1 + :4 v4 :5 v7 2 / :7)
## C8 : delay between two applemen release by the appshoot GOBs.

c0.print "level "
c4.print "apples "
c3.print "tries "
input "appleman.cmd"
import state9

## A first way to have applemen in the level is to instanciate them explicitly in the 
##  level .cmd file as below. Note the 'e-for-EVIL' as object cast.
gob2 :state9 (160, 128) e
gob2 := (40 :0)  ## sets initial horizontal speed. Required for applemen.

## But the general way to have appleman in a level in Apple Assault is that it is shot
##  by an invisible generator, usually located in a tree trunk or offscreen. For this,
##  we first need to select one "external action" (#9) that will generate GOBs.

## ''using <action-name>(<parameters>) as %ACTION_NUMBER%'' is the generic template,
##  and ''shgob'' invokes the GobGunFactory (see guns.cxx). Note that the state number
##  refers to a state as defined within "appleman.cmd", not to the top-level linkage
##  as in ''gob%% :state%%''. Note too that ''is e'' defines the cast (EVIL) of generated
##  gobs.
using shgob(state0 is e:=D) as 9
input "appshoot.cmd"
import state10..11


## and finally, the declaration of the two generators. v5 for the generator 
##  can be modified to de-synchronise the generators.
gob8 :state11 (8, 40) e
gob8 := (1 :5)
# note that those gob% :state%% (<coords>) can be graphically edited with LEDS.
#  http://sylvainhb.blogspot.com/search/label/level%20editor for details.
gob9 :state11 (239, 48) e
gob9 := (2 :5)
gob11 :state13 (120,0) e
gob11:=(10:7 c8 3* 5+ :8 v8:9 80:4)

do fade(0)     # unveils the screen
do track(pop)  # resume the music as it was prior to level change jingle.
end
# here be dragons
# here be dragons
