# -*- sh -*-
# The woodworm is likely the simplest GameObject you can think of. It simply
# moves forward until it detects an obstacle and then turn back. It has no
# hitpoints but can be temporarily disabled by stomping it.
#
# Once you've understood everything about the woodworm, you're suggested to
#  continue your trip in funghi.cmd

# A Gob template .cmd file contains trhee kind of statements: 
# - animation definitions
# - state definitions
# - transitions definitions.
# the script is processed line by lines in a single pass, so every item 
# *MUST* have been defined prior its first use in another statement.

anim1 3 {       # we first have to declare the animations that will be used
  spr0 4        # for this GOB (Game OBject). The spriteset is organised into
  delay 10      # pages by the sprite editor, and each animation may only use
  move 2 0      # sprites held on the same page. The woodworm appears on page
  spr0 5        # 3 of SpriteY.SPR, so we will see ''anim% 3 { <commands> }''
  delay 5       # everywhere in this file.
  move 2 0 
  spr0 6        # note that the "end of line" character is the only meaningful
  delay 10      # separator in GameScript, so no, I'm afraid we can't compact
  spr0 5        # this whole animation on less lines.
  delay 5
  spr0 4        # this one was animation for walking right. Walking left (anim2)
  cloop         # is very similar and will be explained in more detail.
}

anim3 3 {       # basic commands for an animation include changing the picture on
  spr0 4        # screen ''spr0 <sprite-number-on-page-in-hex>'' and requesting a 
  delay 15      # delay before the next operation can take place, so this "animation"
  done          # is just a still image (woodworm waiting) shown for a constant duration
}

anim4 3 {       # the top-bits of the sprite-number-on-page encode the flipping attributes
  spr0 4004     # so spr0 4004 is requesting sprite #4 on the page, but horizontally flipped
  delay 15      # looking left
  done          # delay is in screen retraces, so ''delay 15'' waits 1/4 second.
}

anim2 3 {       # This is the "walking-left" animation for the woodworm. Playing frames
  spr0 4004     # 4-5-6 show the worm's tail extending, and playing 6-5-4 makes it recoil.
  delay 10      # we use an additional ''move <delta-x> <delta-y>'' to alter the GOB's 
  move -2 0     # coordinates. 
  spr0 4005     # for the Woodworm, the ''move'' statement in the anim is the only place
  delay 5       # where we make the GOB move.
  move -2 0
  spr0 4006
  delay 10      # we want this animation to be repeated as long as there is an horizontal
  spr0 4005     # ground to be walked on. The ''cloop'' animation command will restart the
  delay 5       # animation after the "test-points" checks has been succesfully performed.
  spr0 4004     # See below for test-points definition.
  delay 
  cloop
}

anim7 3 {       # woodworm stunned for 60 frames (one second)
  spr0 7
  delay 60
  done
}

###########################################################################################
## Here starts the second part: definition of states.
## States are unit of behaviour for our monsters. Whenever you want the appearance of
## a monster to be changed, its collision areas to be altered or its movements to follow
## different rules, you need a new state. 
## The woodworm quite naturally require one state for walking left and one for walking right,
## (state0 and state1, respectively) but it also has two states for "standing still" 
## for a short duration before it turns back (state2 and state3), and states 4-7 to show the
## worm stunned while remembering the state it had before it got stomped.


state0 :anim1 { # "walking" to the right
  testpoint on (15,16)          # the two testpoints are defined with GOB-relative coordinates.
  testpoint off (16,12)         # the ''testpoint on'' indicates that there should be a solid 
  test 0 (8,2)-(14,14) 0001     # tile below the woodworm and ''testpoint off'' that there should
  area 0 (7,0)-(15,4) 000A      # be a non-solid tile just in front of it.
}                               # you can define up to 4 test-points per state.

state1 :anim2 { # "walking to the left"
  testpoint on (-4,16)          # ''test <area-number> (<top-left>)-(<bottom-right>) <flags>''
  testpoint off (-1,12)         # statement declares an active collision area. At every frame, 
  test 0 (2,2)-(8,14) 0001      # the game engine will look for GOBs that intersect with this
  area 0 (0,0)-(8,4) 000A       # area. The <flags> are freely used by gamescript programmer to
}                               # define the properties of the collision. In this game, 0001 can
# be thought as ''IS_THE_HERO'' property: the worm only "hurts" 
# the hero.

state2 :anim3 { # standing, looking left
  area 0 (7,0)-(15,4) 000A      # ''area <area-number> (<top-left>)-(<bottom-right>) <flags>''
  test 0 (8,2)-(14,14) 0001     # declares a "passive" collision area, against which active areas
}                               # declared with ''test ...'' will be checked. note tha the <flags>
state3 :anim4 {                 # field is always given in hexadecimal, to allow easier combinations
  area 0 (8,0)-(15,4) 000A      # e.g. ''000A'' will receive collisions either from tests carrying
  test 0 (2,2)-(8,14) 0001      # flag 0008 (IS_WEAPON) or flag 0002 (HERO_STOMPING)
}
# since there is no move in "standing" states, there is no use for testpoints.

state4 :anim7 { # nothing happens when the woodworm is stunned on the ground, 
}               # so those states are fairly empty. No collision to issue (it no longer hurts)
state5 :anim7 { # nor to receive (it can't be hurt more)
}
state6 :anim7 {
}

#############################################################################################
## Transition definitions
## This is where the "real" game-logic programming usually takes place. By defining how you
## can switch from one state to the other, under which condition and with what side-effect, 
## you actually build the behaviour of your GOB. 
##
## Every transition must mention one (or several) "input state" to which it applies, one
## "output state" that should define the new behaviour of the GOB, the type of event that
## triggers this transition, an optional guard expression and an optional list of action.
##
## state{in}->state{out} on {event-type} [condition] (action)

## the core behaviour of the woodworm consists of walking as far as possible and then turning
## back, this is achieved with the following transitions:
state0->state2 on fail
state1->state3 on fail
state2->state1 on done
state3->state0 on done
## the ''fail'' event-type indicates that the ''cloop'' (or ''check'') animation command has
## been processed, but that the testpoint check failed -- for instance, there was a wall in front
## of the worm.
## The ''done'' event-type simply indicates that the animation has been played completely, up to
## the ''done'' animation command.

## to complete the behaviour of our woodworm, we also want him to be stunned if it is stomped
## or otherwise attacked by the hero. 
state0..1->state4 on hit0 [t]
state2->state5 on hit0 [t]
state3->state6 on hit0 [t]
# First, we use the ''hit%'' event-type to indicate that switching to one of the stunned state
# will occur when another gob has a collision test that intersects with our
# ''area 0 (X,0)-(X+8,4) 000A'' passive area. the digit after ''hit'' must match the number of 
# the area. 

# "staying stumped for a short time" is achieved here through the ''delay 60'' command in the
# animation list for states 4-6. This commands accepts a delay up to 255. Longer delays can be
# realised through a sequence of ''delay'' commands, or using internal variables, as we will see
# in funghi.cmd ... See you there.
state4->state0 on done
state5->state2 on done
state6->state3 on done

end # every command file must be terminated by an explicit "end" command, 
    # anything beyond this command is ignored by the script reader.
